import javax.vecmath.*;

/**
 * Title: DistanceMeter
 * Description: This class is used to measure the distance between a given area
 * and a Point
 * @author Dimitri Missoh, Daniel Schlosser
 * @version 1.0
 */

public class DistanceMeter extends java.lang.Object
{
  private double[] values=new double[3];
  private double[] results=new double[3];
  private double[] faktoren=new double[4];
  private double[] backup=new double[3];
  private int[] indexA=new int[3];
/**
 * Creates a DistanceMeter using 3 points to calculate a 2 dimensional Area
 * in 3d worldspace. Then we calculate some numbers to calculate the distance
 * of a given point without solving the equation system
 * @param a vector describing first point
 * @param b vector describing second point
 * @param c vector describing third point
 */
  public DistanceMeter(Vector3d a,Vector3d b, Vector3d c)
  {
    for(int i=0;i<3;i++)
      indexA[i]=i;
    Vector3d ab=new Vector3d();
    ab.set(b);
    ab.sub(a);
    Vector3d bc=new Vector3d();
    bc.set(c);
    bc.sub(b);
    if(testlinearab(ab,bc)) throw new RuntimeException("Vektoren linear abhngig");
    Vector3d n=new Vector3d();
    n.set(ab);
    n.cross(ab,bc);
    n.normalize();
    double[][] A=new double[3][3];
    A[0][0]=ab.x;
    A[1][0]=ab.y;
    A[2][0]=ab.z;
    A[0][1]=bc.x;
    A[1][1]=bc.y;
    A[2][1]=bc.z;
    A[0][2]=n.x;
    A[1][2]=n.y;
    A[2][2]=n.z;
    double max=0;
    int Zeile=0;
    for(int i=0;i<3;i++)
      if(Math.abs(A[i][0])>max)
      {
        Zeile=i;
        max=Math.abs(A[i][0]);
      }
    //Zeile gespeichert
    if(Zeile!=0)
    {//Zeilen vertauschen
      for(int i=0;i<3;i++)
      {
        max=A[0][i];
        A[0][i]=A[Zeile][i];
        A[Zeile][i]=max;
      }
      indexA[Zeile]=0;
      indexA[0]=Zeile;
    }
    //Wenn MatrixE ungleich null 1. GaussSchritt
    for(int j=1;j<3;j++)
    {
        if(A[j][0]!=0)
        {
            A[j][0]/=A[0][0];
            A[j][1]-=A[0][1]*A[j][0];
            A[j][2]-=A[0][2]*A[j][0];
        }
    }
    //Pivotsuche 2
    max=0;
    for(int i=1;i<3;i++)
    {
      if(Math.abs(A[i][1])>max)
      {
        max=Math.abs(A[i][1]);
        Zeile=i;
      }
    }
    if(Zeile==2)
    {//Wenn echtes Pivot dann vertauschen
      for(int i=0;i<3;i++)
      {
        max=A[1][i];
        A[1][i]=A[2][i];
        A[2][i]=max;
      }
      int puffer=indexA[1];
      indexA[1]=indexA[2];
      indexA[2]=puffer;
    }
    //2. GaussSchritt
    if(A[2][1]!=0)
    {
      A[2][1]/=A[1][1];
      A[2][2]-=A[1][2]*A[2][1];
    }
    // Fertig jetzt werte Abgreifen
    faktoren[0]=c.x;
    faktoren[1]=c.y;
    faktoren[2]=c.z;
    for(int i=0;i<3;i++)
      values[i]=faktoren[indexA[i]];
    faktoren[0]=A[1][0];
    faktoren[1]=A[2][0];
    faktoren[2]=A[2][1];
    faktoren[3]=A[2][2];
  }
  /**
   * This function tests if the two given vectors are pointing to
   * the same direction
   */
  private boolean testlinearab(Vector3d a, Vector3d b)
  {
    a.normalize();
    b.normalize();
    if(a.x==b.x)
	if ( (a.y==b.y) && (a.z==b.z) )
	    return true;
    else
    {
	a.scale(-1);
	if(a.x==b.x && a.y==b.y && a.z==b.z)
	    return true;
    }
      return false;
  }
  /**
   * This function calculates the distance between a given point and the area used
   * to create DistanceMeter
   * @param ov the 'Ortsvektor' of the point
   */
  public double getDistance(Vector3d ov)
  {
    backup[0]=-ov.x;
    backup[1]=-ov.y;
    backup[2]=-ov.z;
    for(int i=0;i<3;i++)
      results[i]=values[i]+backup[indexA[i]];
    return (results[2]-results[0]*faktoren[1]-(results[1]-results[0]*faktoren[0])*faktoren[2])/faktoren[3];
  }
}
