/**
 * Klasse Gen, die eine einfache Strucktur fr ein Gen zur Verfgung stellt.
 *
 * Creation date 29.05.2001
 * @author Bernd Jkel and Susanne Halstead
 * @version 1.1
 * @see Genom, GenPool
 */

import java.lang.*;
import java.util.*;


public class Gen
{
	/**
	 * Integer Array der das Sichtfeld der Ameise darstellt
	 */
	private char[][][] pattern;
	/**
	 * Integer View der die Blickweite des Gens darstellt
	 */
	private int view;
	/**
	 * Integer Direction das die Bewegungsrichtung der Ameise nach dem Bau ausgibt
	 */
	private int direction;


	/**
	 * Standard Konstruktor der ein Gen mit lnge x, breite y und hhe z schafft.
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @param view_in int der lnge des Blickfeldes darstellt
	 * @param probabilityStein double der die Wahrscheinlichkeit fr einen Stein darstellt
	 * @param probabilityJoker double der die Wahrscheinlichkeit fr einen Joker darstellt
	 * @param probabilityLeer double der die Wahrscheinlichkeit fr einen leeren Block darstellt
	 * @return Gen ein mit den Werten initialisiertes Gen
	 */
	public Gen(int view_in, double probabilityStein,  double probabilityJoker, double probabilityLeer) {
		pattern = new char[view_in][view_in][view_in];
		view = view_in;
		boolean bernd = true;
		int stein;

		while (bernd) {
			for (int i = 0; i < view_in; i++) {
				for (int j = 0; j < view_in; j++) {
					for (int k = 0; k < view_in; k++) {
						double random_ = Math.random() * (probabilityStein + probabilityJoker + probabilityLeer);
						if ((random_ - probabilityStein) < 0) {
							stein = 1;
												} else if ((random_ - probabilityStein - probabilityJoker) < 0) {
							stein = -1;
												} else if ((random_ - probabilityStein - probabilityJoker - probabilityLeer) < 0) {
							stein = 0;
						} else {
							stein = 0;
						}
						if (stein == 1) {
							bernd = false;
						}
						pattern[i][j][k] = makeBlock(stein);
					}
				}
			}

                                        pattern[view_in / 2][view_in / 2][view_in / 2] = makeBlock(1);
			direction =  (int)Math.abs(Math.round(Math.random() * ((view * view * view) -1 )) % ((view * view * view) -1 ));
		}
	}

	/**
	 * Methode die einen eingegebenen int i in einen char umwandelt der dann eine
	 * Stelle im Gen darstellt.
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param i int eingegebener int
	 * @return char der den Block zu diesem int i darstellt
	 */
	public char makeBlock(int i) {
		if (i == 0) {
		return Nest.LEER;
	} else if (i == 1) {
		return Nest.STEIN;
	} else if (i == -1) {
		return Nest.JOKER;
	}
	return Nest.LEER;
	}

	/**
	 * Konstruktor der ein Gen mit lnge x, breite y und hhe z schafft, dass eine einfache Belegung
	 * hat, so dass sie Ameise mit diesem Gen das Bauen beginnen kann.
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @param view_in int lnge des Blickfeldes
	 * @return Gen ein mit diesen Werten x, y, z initialisiertes Gen
	 */
	public Gen(int view_in) {
		pattern = new char[view_in][view_in][view_in];

		for (int i = 0; i < view_in; i++) {
			for (int j = 0; j < view_in; j++) {
				for (int k = 0; k < view_in; k++) {
					pattern[i][j][k] = Nest.LEER;
				}
			}
		}


		view = view_in;

				int a = view_in / 2,b = view_in / 2 , c = view_in/ 2;
				while ((a == view_in/2) && (b == view_in/2) && (c == view_in/2)) {

						a = (int)Math.abs(Math.round(Math.random()) % view_in);
						b = (int)Math.abs(Math.round(Math.random()) % view_in);
						c = (int)Math.abs(Math.round(Math.random()) % view_in);

						pattern[a][b][c] = Nest.JOKER;
				}

				a = view_in / 2;
				b = view_in / 2;
				c = view_in / 2;
				while ((a == view_in/2) && (b == view_in/2) && (c == view_in/2)) {

						a = (int)Math.abs(Math.round(Math.random()) % view_in);
						b = (int)Math.abs(Math.round(Math.random()) % view_in);
						c = (int)Math.abs(Math.round(Math.random()) % view_in);

						pattern[a][b][c] = Nest.STEIN;

				}


		pattern[view_in / 2][view_in / 2][view_in / 2] = Nest.STEIN;
		direction =  (int)Math.abs(Math.round(Math.random() * ((view * view * view) -1)) % ((view * view * view) -1));
	}

	/**
	 * Konstruktor der ein Gen mit lnge x, breite y und hhe z aus einem String schafft.
	 * Creation date 31.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @param x int lnge des Blickfeldes
	 * @param y int breite des Blickfeldes
	 * @param z int hhe des Blickfeldes
	 * @param gen Sting der das Gen reprsentiert
	 * @return Gen ein mit diesen Werten x, y, z initialisiertes Gen
	 */
	public Gen(String gen) {
		StringTokenizer st = new StringTokenizer(gen);


		String hier = st.nextToken();

		int x = hier.length();

		if (x == 9) {
			x = 3;
		} else if (x == 25) {
			x = 5;
		} else if (x == 49) {
			x = 7;
		}

		pattern = new char[x][x][x];

		view = x;

                for (int i = 0; i < view; i++) {
			for (int j = 0; j < view; j++) {
				for (int k = 0; k < view; k++) {
                                        pattern[k][j][i] = hier.charAt(k * 3 + j);
				}
			}
			hier = st.nextToken();
		}

                Integer in = new Integer(hier);
		direction = in.intValue();
	}

	/**
	 * Methode die ein Gen in einen String umwandet
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @return String der das Gen Darstellt
	 */
	public String toString() {
		String ret = new String();

		for (int i = 0; i < view; i++) {
			for (int j = 0; j < view; j++) {
				for (int k = 0; k < view; k++) {
					ret = ret.concat((new Character(pattern[j][k][i])).toString());
				}
			}
			ret = ret.concat(" ");
		}

		ret = ret.concat(" ");
		ret = ret.concat((new Integer(getDirection())).toString());
		ret = ret.concat(Nest.NEWLINE);
		return ret;
	}

	/**
	 * Methode die die Hhe eines Gens zurckliefert
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.0
	 * @return Hhe des Gens
		 */
	public int getView() {
		return view;
	}

	/**
	 * Methode die das Pattern eines Gens zurckliefert
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @return Pattern des Gens
	 */
	public char[][][] getPattern() {
		return this.pattern;
	}

	/**
	 * Methode die die Richtung der Ameise nach dem Bau zurckliefert
	 * Creation date 29.05.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.1
	 * @return Direction des Gens
	 */
	public int getDirection() {
		return direction;
	}

	/**
	 * Methode die zurck gibt, ob ein Gen bei entsprechendem Pattern bauen anstoen soll, oder nicht
	 * Creation date 01.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @return boolean der angibt ob gebaut werden soll, oder nicht
	 */
	public boolean getBuild() {
		if (pattern[view / 2][view / 2][view / 2] == Nest.LEER) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Methode die einen Stein eines Patterns auf einen Bestimmten Wert setzt.
	 * Creation date 01.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param x int die x Koordinate des Steins
	 * @param y int die y Koordinate des Steins
	 * @param z int die z Koordinate des Steins
	 * @param c chat der Stein der an die angegebene Stelle gestetzt wird
	 */
	public void setPattern(int x, int y, int z, char c) {
		pattern[x][y][z] = c;
	}

	/**
	 * Methode die einen Stein eines Patterns an einer bestimmten stelle zurckgibt
	 * Creation date 01.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param x int die x Koordinate des Steins
	 * @param y int die y Koordinate des Steins
	 * @param z int die z Koordinate des Steins
	 * @return chat der Stein der an der angegebene Stelle
	 */
	public char getPattern(int x, int y, int z) {
		return pattern[x][y][z];
	}


	/**
	 * Methode die die Richtung der Ameise nach dem bauen verndert
	 * Creation date 01.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param direction_in int die neue Richtung
	 */
	public void setDirection(int direction_in) {
		direction = direction_in;
	}

	public Gen cross(Gen gen, int place) {

		Gen genout = new Gen(gen.getView());

		for (int i = 0; i < view; i++) {
			for (int j = 0; j < view; j++) {
				for (int k = 0; k < view; k++) {
					if (place > 0) {
						genout.setPattern(j, k, i, getPattern(j, k , i));
						place--;
					} else {
						genout.setPattern(j, k, i, gen.getPattern(j, k , i));
					}
				}
			}
		}
		if (place > 0) {
			genout.setDirection(getDirection());
		} else {
			genout.setDirection(gen.getDirection());
		}
		return genout;
	}
}
