/**
 * Stellt eine LinkedList, die ein GenPool reprsentiert, dar
 *
 * Creation date 05.07.2001
 * @author Bernd Jkel and Susanne Halstead
 * @version 1.3
 * @see Gen, Genom
 */

import java.util.LinkedList;

public class GenPool extends LinkedList
{
	/** Konstruktor der einen neuen GenPool erzeugt
	 *
	 * Creation 15.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 */
	public GenPool() {
	}

	/**
	 * Konstruktor der einen neuen GenPool mit numberOfGenoms Genomen, die je numberOfGens Gene haben,
	 * und gewichtetem Auftreten von Stein, Joker und Leer
	 *
	 * Creation 15.06.2001
	 * @author Bernd Jkel und Susanne Halstead
	 * @version 1.3
	 * @param int numberOfGenoms die Aanzahl der Genome in einem GenPool
	 * @param int numberOfGens die Anzahl der Gene in einem GenPool
	 * @param view int Sichtweite der Ameise
	 * @param probabilityStein double Gewicht fr Stein
	 * @param probabilityJoker double Gewicht fr Joker
	 * @param probabilityLeer double Gewicht fr leere Stelle
	 */
	public GenPool(int numberOfGenoms, int numberOfGens, int view, double probabilityStein,  double probabilityJoker, double probabilityLeer) {
		for (int i = 0; i < numberOfGenoms; i++) {
			add(new Genom(numberOfGens, view, probabilityStein, probabilityJoker, probabilityLeer));
		}
	}

	/**
	 * Konstruktor der einen neuen GenPool einem Genom
	 *
	 * Creation 15.06.2001
	 * @author Bernd Jkel und Susanne Halstead
	 * @version 1.3
	 * @param Genom genom das einzufgende Genom
	 * @param double fitness der FitnessValue des Genoms
	 */
	public GenPool(Genom genom, double fitness) {
		genom.setFitness(fitness);
		add(genom);
	}

	/**
	 * Fgt ein Genom in den Genpool ein
	 *
	 * Creation 10.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 * @param Genom genom das einzufgende Genom
	 * @param double fitness der FitnessValue des Genoms
	 */
	public void add(Genom genom, double fitness) {
		genom.setFitness(fitness);
		add(genom);
	}

	/**
	 * Gibt aus dem GenPool das Genom an stelle i zurck
	 *
	 * Creation 10.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 * @param int i die Stelle an der das Genom ist
	 * @return Genom an Stelle i
	 */
	public Genom getGenom(int i) {
		Genom hier = (Genom)get(i);
		return (Genom)hier;
	}

	/**
	 * Setzt der FitnessValue eines Genoms auf einen bestimmten Wert
	 *
	 * Creation 10.06.2001
	 * @version 1.3
	 * @author Bernd Jekel und Susanne Halstead
	 * @param int place die Stelle des Genoms
	 * @param double value der den FitnessValue darstellt
	 */
	public void setValue(int place, double value) {
		Genom hier = (Genom)get(place);
		hier.setFitness(value);
	}

	/**
	 * Macht den Crossoverprozess mit einem GenPool und gibt einen neuen GenPool zurck
	 *
	 * Creation 10.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 * @param int numberOfGenoms die gre des GenPools der dabei entsteht
	 * @return GenPool der aus dem alten GenPool durch Crossover entstand
	 */
	public GenPool doCrossover(int numberOfGenom) {
                doCrossover(numberOfGenom, (int)((size() * 10) / 9));
		return this;
	}


	/**
	 * Macht den Crossoverprozess mit einem GenPool und gibt einen neuen GenPool zurck
	 *
	 * Creation 10.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 * @param int numberOfGenoms die gre des GenPools der dabei entsteht
	 * @param int numberOfselektion die gre des GenPools der benutzt werden soll
	 * @return GenPool der aus dem alten GenPool durch Crossover entstand
	 */
	public GenPool doCrossover(int numberOfGenom, int numberOfSelection) {
		GenPool out = new GenPool();
		Genom genom1 = new Genom();
		Genom genom2 = new Genom();


		if (!((numberOfSelection == -1) || (numberOfSelection > size()) || (numberOfSelection < 2))) {
			for (int i = 0; i < size(); i++) {
				genom1 = (Genom)get(i);
				boolean susanne = true;

				if (out.size() < numberOfSelection) {
					out.add(genom1);
				} else {
					for (int j = 0; (j < numberOfSelection) && susanne; j++) {
						genom2 = (Genom)get(j);
						if (genom2.getFitness() < genom1.getFitness()) {
							out.remove(j);
							out.add(j, genom1);
							susanne = false;
						}
					}
				}
			}

			while (size() > 0) {
				remove(0);
			}
			while (out.size() > 0) {
				add(out.get(0));
				out.remove(0);
			}
			out = new GenPool();
		}

		int a = -1;
		int b = -1;
		double sum = 0;

		for (int i = 0; i < size(); i++) {
					Genom hier = (Genom)get(i);
					sum = sum + hier.getFitness();
		}

		for (int i = 0; i < numberOfGenom; i++) {
			Genom genomout = new Genom();


			while (a == b) {
				a = -1;
				b = -1;

				double r1 = (Math.random() * sum);

				while (r1 > 0) {
					a++;
					genom1 = (Genom)get(a);
					r1 = r1 - genom1.getFitness();
				}

				double r2 = (Math.random() * sum);


				while (r2 > 0) {
					b++;
					genom2 = (Genom)get(b);
					r2 = r2 - genom2.getFitness();
				}

			}

			int genLaenge = (((Gen)genom1.get(0)).getView() * ((Gen)genom1.get(0)).getView() * ((Gen)genom1.get(0)).getView()) + 1;

			int schnittStelle = 0;

			while (schnittStelle < genLaenge) {
				int min = (int)Math.min(genom2.size(), genom1.size());
				int z111 = (int)Math.round(Math.abs(Math.random() * (double)genLaenge));
				schnittStelle = (int)(min * (z111));
			}

			int gen_1 = 0;

			while (schnittStelle - genLaenge > 0) {
				genomout.add(genom1.get(gen_1));
				gen_1++;
				schnittStelle = schnittStelle - genLaenge;
			}

			Gen gen1 = (Gen)genom1.get(gen_1);
			Gen gen2 = (Gen)genom2.get(gen_1);
			Gen gen = gen1.cross(gen2, schnittStelle);

			genomout.add(gen);
			gen_1++;

			while (gen_1 < genom2.size()) {
				genomout.add(genom2.get(gen_1));
				gen_1++;
			}

			out.add(genomout);
		}
		while (size() > 0) {
			remove(0);
		}
		while (out.size() > 0) {
			add(out.get(0));
			out.remove(0);
		}

		return this;
	}

	/**
	 * Funktion die einen GenPool Mutieren lt
	 *
	 * Creation date 10.06.2001
	 * @version 1.3
	 * @authors Bernd Jkel und Susanne Halstead
	 * @param double probability die Wahrscheinlichkeit mit der Mutiert werden soll (zwischen 0 und 1)
	 * @return GenPool den mutierten GenPool
	 */

	public GenPool doMutation(double probability) {
		GenPool out = new GenPool();

				if (probability > 1) {
						probability = probability / 100;
				}

		for (int i = 0; i < size(); i++) {
			Genom genom_out = new Genom(), genom = (Genom)get(i);

			for (int j = 0; j < genom.size(); j++) {
				Gen gen = (Gen)genom.get(j);

				int laenge = gen.getView();
				int breite = gen.getView();
				int hoehe = gen.getView();

								for (int ix = 0; (ix < laenge) && (j != 0); ix++) {
					for (int iy = 0; iy < breite; iy++) {
						for (int iz = 0; iz < hoehe; iz++) {
							if (Math.random() < probability) {
								int stein = 0;
								double mr = Math.random();
								if (mr < 0.7) {
									stein = 0;
								} else if (mr < 0.95){
									stein = -1;
								} else {
									stein = 1;
								}

								gen.setPattern(ix, iy, iz, gen.makeBlock(stein));
							}
						}
					}
				}

				if (Math.random() < probability) {
					gen.setDirection((int)Math.abs(Math.round(Math.random() * 26) % 26));
				}

				genom_out.add(gen);

			}
			genom_out.setFitness(genom.getFitness());
			out.add(genom_out);
		}
		return out;
	}

	/**
	 * Funktion die einen GenPool in einen String umwandelt
	 *
	 * Creation date 10.06.2001
	 * @version 1.3
	 * @author Bernd Jkel und Susanne Halstead
	 * @return der String der den GenPool darstellt
	 */

	public String toString() {
		String out = new String();

		for(int i = 0; i < size(); i++) {
			Genom genom = (Genom)get(i);
			out = out.concat(genom.toString());
			out = out.concat(Nest.NEWLINE);
		}

		return out;
	}
}
