/**
 * Klasse Genom, die eine einfache Strucktur fr ein Genom zur Verfgung stellt.
 *
 * Creation date 29.05.2001
 * @author Bernd Jkel and Susanne Halstead
 * @version 1.0
 * @see Gen, GenPool
 */

import java.lang.*;
import java.util.LinkedList;


public class Genom extends LinkedList
{
	/**
	 * Der Fitnessvalue des Genoms, wird am Anfang mit 0 initialisiert
	 *
	 * Creation date 14.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 */
	private double fitness = 0;

	/**
	 * Erzeugt eine leere LinkedList, die ein Genom reprsentiert.
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 */
	public Genom() {
	}

	/**
	 * Erzeugt eine leere LinkedList, die ein Genom reprsentiert. Mit number Genen
	 * Und gewichtentem Auftreten von Stein, Leer und Joker
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param number int Anzahl der Gene in einem Genom
	 * @param view int Sichtweite der Ameise
	 * @param probabilityStein double Gewicht fr Stein
	 * @param probabilityJoker double Gewicht fr Joker
	 * @param probabilityLeer double Gewicht fr leere Stelle
	 */
	public Genom(int number, int view, double probabilityStein,  double probabilityJoker, double probabilityLeer) {
				add(new Gen(view));

		for (int i = 0; i < number - 1; i++) {
			add(new Gen(view, probabilityStein,  probabilityJoker, probabilityLeer));
		}
	}

	/**
	 * Erzeugt ein Genaom aus eingelesenen Genen
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param gens String[] Gen in Stringform
	 */
	public Genom(String[] gens ) {
		for (int i = 0; i < gens.length; i++) {
			add(new Gen(gens[i]));
		}
	}

	/**
	 * Wandelt ein Genom in einen String um
	 *
	 * Creation date 01.06.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.2
	 * @return String der das Genom Darstellt
	 */
	public String toString() {
		String ret = new String();
		for (int i = 0; i < size(); i++) {
			ret = ret.concat(get(i).toString());
		}
		return ret;
	}

	/**
	 * Gibt das Pattern in der fr die Ameise lesbaren Form aus
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param gen int fr das Gen an Stelle gen
	 * @return char[] der das Pattern als char[] darstellt
	 */
	public char[] getPatternOfGen(int gen) {
		Gen g = (Gen)get(gen);

		int view = g.getView();

		char pattern[][][] = g.getPattern();
		char out[] = new char[view * view * view];

		for (int i = 0; i < view; i ++) {
			for (int j = 0; j < view; j ++) {
				for (int k = 0; k < view; k ++) {
                                out[(i * view * view) + (j * view) + k] = pattern[j][k][i];
				}
			}
		}

		out[((view * view * view) -1) / 2] = Nest.LEER;

		return out;

	}


	/**
	 * Gibt die Richtung von Gen i des Genoms an
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param int i fr das Gen an Stelle i
	 * @return int der die Richtung darstellt
	 */
	public int getDirectionOfGen(int i) {
		Gen g = (Gen)get(i);
		return g.getDirection();
	}

	/**
	 * Setzt die Fitness eines Genoms auf einen bestimmten Wert
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param float fitness_IN die Fitness des Genoms
	 */
	public void setFitness(double fitness_IN) {
		fitness = fitness_IN;
	}

	/**
	 * Gibt die Fitness eines Genoms raus
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @return float fitness die Fitness des Genoms
	 */
	public double getFitness() {
		return fitness;
	}

	/**
	 * Gibt an ob fr Gen i gebaut werde mu oder nicht
	 *
	 * Creation date 01.01.2001
	 * @author Bernd Jkel and Susanne Halstead
	 * @version 1.3
	 * @param int i fr das Gen an Stelle i
	 * @return boolean ob gebaut werden soll oder nicht
	 */
	public boolean getBuildOfGen(int i) {
		Gen g = (Gen)get(i);
		return g.getBuild();
	}
}
