import javax.vecmath.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
/**
 * Title: MyUniverse
 * Description: MyUniverse is a virtual universe with one directional light right
 * over the watchers eyes
 * @author Dimitri Missoh & Daniel Schlosser
 * @version 0.1
 */

public class MyUniverse extends VirtualUniverse {
       private static final double STANDARTDISTANCE=101.0d;
       private double xAngle,yAngle,zAngle;
       private Locale locale;
       private BranchGroup ViewGroup;
       private TransformGroup viewZoomTG,viewRotateTG,viewCenterTG;
       private Transform3D viewTransfRo, pufferTransfRo,viewTransfTr,viewTransfZo;
       private Vector3d viewTransVec;
       private ViewPlatform platform;
       private View view;
       private DirectionalLight headlight;
       private PhysicalBody body;
       private PhysicalEnvironment env;
       private Box visBox;
       private Vector3d pufferVec;
       private TheHive theHive;
       private boolean mouseRotation=false;

    public MyUniverse(Canvas3D cv,TheHive TH) {
	   theHive=TH;
	   pufferVec=new Vector3d();
           viewTransfRo=new Transform3D();
	   viewTransfZo=new Transform3D();
	   viewTransfTr=new Transform3D();
           pufferTransfRo=new Transform3D();
           locale=new Locale(this);
           ViewGroup=new BranchGroup();
            //Konstruktion eines Lichtes ber dem POV
           headlight=new DirectionalLight();
           headlight.setCapability(Light.ALLOW_STATE_WRITE);
           headlight.setColor(new Color3f(1.0f,1.0f,1.0f));
           headlight.setInfluencingBounds(new BoundingSphere(new Point3d(0.0,0.0,0.0),1000.0));
           //erzeuge Instanzen fr View
           body=new PhysicalBody();
           env=new PhysicalEnvironment();
           platform=new ViewPlatform();
           view=new View();
           view.setBackClipDistance(1000.0);
           view.setPhysicalBody(body);
           view.setPhysicalEnvironment(env);
           view.attachViewPlatform(platform);
           view.addCanvas3D(cv);
           //bringe Camera in Standartabstand
           viewTransVec=new Vector3d(0.0,0.0,STANDARTDISTANCE);
           viewZoomTG=new TransformGroup();
           viewZoomTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
           viewZoomTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
           viewTransfZo.setTranslation(viewTransVec);
           viewZoomTG.setTransform(viewTransfZo);
           viewZoomTG.addChild(headlight);
           viewZoomTG.addChild(platform);
           // aus Richtung 0
           viewRotateTG=new TransformGroup();
           viewRotateTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
           viewRotateTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
           viewTransfRo.setIdentity();
           viewRotateTG.setTransform(viewTransfRo);
           viewRotateTG.addChild(viewZoomTG);
           xAngle=0;
           yAngle=0;
           zAngle=0;
           // mit Mittelpunkt 0,0,0
           viewCenterTG=new TransformGroup();
           viewCenterTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
           viewCenterTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
           viewTransVec.set(0.0,0.0,0.0);
           viewTransfTr.setTranslation(viewTransVec);
           viewCenterTG.setTransform(viewTransfTr);
           viewCenterTG.addChild(viewRotateTG);
           //und View add to Universe
           ViewGroup.addChild(viewCenterTG);
           locale.addBranchGraph(ViewGroup);
    }
    public void rotateXAbsolut(double angle)
    {
            this.mouseRotation=false;
            xAngle=angle;
            doRotation();
    }
    public void rotateYAbsolut(double angle)
    {
            this.mouseRotation=false;
            yAngle=angle;
            doRotation();
    }
    public void rotateZAbsolut(double angle)
    {
            this.mouseRotation=false;
            zAngle=angle;
            doRotation();
    }

    private void doRotation()
    {
          viewTransfRo.setEuler(new Vector3d(xAngle,yAngle,zAngle));
          viewRotateTG.setTransform(viewTransfRo);
    }

    public void rotateRelativToView(int dx, int dy)
    {
	viewRotateTG.getTransform(viewTransfRo);
	pufferTransfRo.rotY(dx/180.0*Math.PI);
        viewTransfRo.mul(pufferTransfRo);
        pufferTransfRo.rotX(dy/180.0*Math.PI);
        viewTransfRo.mul(pufferTransfRo);
        viewRotateTG.setTransform(viewTransfRo);
        if(!this.mouseRotation)
        {
            theHive.setRotationValueViewOffline();
            this.mouseRotation=true;
        }
    }

    public void translateCenterAbsolut(int x,int y,int z)
    {
            viewTransVec.set(x,y,z);
            viewTransfTr.setTranslation(viewTransVec);
            viewCenterTG.setTransform(viewTransfTr);
	    theHive.setTranslationState(x,y,z);
    }
    public void zoomAbsolut(double distance){
           viewTransfZo.setTranslation(new Vector3d(0.0,0.0,distance));
           viewZoomTG.setTransform(viewTransfZo);
    }

    public void zoomRelativ(int dy)
    {
        viewTransfZo.get(viewTransVec);
	double length=viewTransVec.length();
	length=length/100.0*dy;
	pufferVec.sub(viewTransVec,new Vector3d(0.0,0.0,length));
	if((pufferVec.length())>1.0)
	    viewTransVec.set(pufferVec);
	viewTransfZo.setTranslation(viewTransVec);
	viewZoomTG.setTransform(viewTransfZo);
	theHive.setZoomState(100/(viewTransVec.length()-1.0));

    }
    public void addBranchGraph(BranchGroup bg)
    {
        locale.addBranchGraph(bg);
    }
}